;; s.t.o.3/theme.jl

;; by Nicholas Riley - comments to <sawfish@sabi.net>
;; based on the s.t.o Sawfish theme, by Michele Campeotto <golia@supereva.it>

;; changes of 27 July 2000:
;; - configuration interface for option settings
;; - live adjustment of settings, no need to restart sawfish
;; - better handling of borders on top side (but see *border-inset*)
;; - support for transient, shaped and shaped transient frames

;; changes of 28 March 2000:
;; - a readable color for non-focused window titles
;; - close box on the left, collapse (windowshade) box on the right
;; - non-hardcoded font setting
;; - border permitting resizing, and (optionally) resizing corners
;; - easily user-adjustable colors/widths of many of the above features

;; -- user configurable settings --

(defgroup sto3 "s.t.o.3 Theme"
  :group appearance)

(defcustom sto3:border-color
  (get-color "#01f001f001f0")
  "Window border color."
  :type color
  :user-level novice
  :group (appearance sto3))

(defcustom sto3:border-width
  2
  "Window border width (in pixels)."
  :type number
  :range (1 . 10)
  :user-level novice
  :group (appearance sto3))

(defcustom sto3:use-corner
  t
  "Use window sizing corner."
  :type boolean
  :user-level novice
  :group (appearance sto3))

(defcustom sto3:corner-color-enabled
  (get-color "#52527272a3a3")
  "Window sizing corner color, when window has focus."
  :type color
  :group (appearance sto3))


(defcustom sto3:corner-color-disabled
  (get-color "#6d6d6d6d6d6d")
  "Window sizing corner color, when window lacks focus."
  :type color
  :group (appearance sto3))

(defcustom sto3:corner-side
  10
  "Length/width of sizing corner (in pixels, added to window border width)."
  :type number
  :range (10 . 20)
  :group (appearance sto3))

(defcustom sto3:corner-border-width
  2
  "Border width of sizing corner (2 and 4 look good)."
  :type number
  :range (1 . 20)
  :group (appearance sto3))

(defcustom sto3:closebox-offset
  12
  "Close box offset (may want to change with font)."
  :type number
  :range (12 . 30)
  :group (appearance sto3))

;; how far should top border intrude into title bar? 0 and 1 are safe choices, it's
;; an aesthetic decision
(defvar *border-inset* 1)

;; these should be 'defconst', but for some reason (safety?) make-binding-immutable
;; isn't defined?

(defvar *top* (- *border-inset* 25))

;; functions - this part doesn't work on 0.26 and earlier
;; you should be able to move these in to the let* if you want...

(defun title-width (w)
  "Return the width of the window title."
  (let ((w-width (car (window-dimensions w))))
    (max 0 (+ 20 (min (- w-width 55) (text-width (window-name w)))))))

(defun corner-border-width ()
  "Return the corner border width, or 0 if sizing corner disabled."
  (if sto3:use-corner sto3:corner-border-width 0))

(defun corner-colors ()
  "Return a list of the disabled and enabled sizing corner colors."
  (list sto3:corner-color-disabled sto3:corner-color-enabled))

(let*
    ((titlebar-left (list (make-image "tl-i.png") (make-image "tl.png")))
     
     (titlebar-right (list (make-image "tr-i.png") (make-image "tr.png")))

     (title-colors (list "LightSteelBlue1" "white"))
     
     (middle (list (make-image "middle-i.png")
                   (make-image "middle.png")))
     
     (corner (list (make-image "corner-i.png")
                         (make-image "corner.png")))

     (close (list (make-image "close.gif")
                  (make-image "close.gif") nil
                  (make-image "close.gif")))

     (maximize (list (make-image "up-i.png")
		     (make-image "up.png") nil
                     (make-image "up.png")))

     (iconify (list (make-image "down-i.png")
			(make-image "down.png") nil
                    (make-image "down.png")))

     (shade (list (make-image "shade-i.png")
                  (make-image "shade.png") nil
                  (make-image "shade.png")))

     (menu (list (make-image "right-i.png")
                 (make-image "right.png") nil
                 (make-image "right.png")))

     ;; thunks and friends
     (closebox-offset (lambda () sto3:closebox-offset))
     (border-color (lambda () sto3:border-color))
     (border-width (lambda () sto3:border-width))
     (-border-width (lambda () (- sto3:border-width)))
     (-corner-border-width (lambda () (- (corner-border-width))))
     (ce nil) (corner-edge (lambda () ce))
     (te nil) (top-edge (lambda () te))
     (cs nil) (corner-side (lambda () cs))
     (titlebar-top (lambda () (- *top* sto3:border-width)))
     (corner-top (lambda () (- *top* (corner-border-width))))

     ;; rebuild after configuration changes
     (init
      (lambda ()
	(setq ce (if (or (< sto3:corner-side 1)
			 (< (corner-border-width) 1))
		     (-border-width) sto3:corner-side))
	(setq cs (+ ce (corner-border-width)))
	(setq te (+ *top* ce))))

     (rebuild
      (lambda ()
	(init)
	(rebuild-frames-with-style 's.t.o.3)))
  
     ;; layout generation
     (make-border
      (lambda (class)
	"Return the window border of the specified class."
	(let ((ib (eq class 'title))
	      (tb (eq class 'top-border)) (bb (eq class 'bottom-border))
	      (lb (eq class 'full-left-border)) (rb (eq class 'full-right-border))
	      (1b (eq class 'left-border)) (2b (eq class 'right-border)))
	  (list `(background . ,border-color)
		(cond ((or ib tb bb) `(left-edge . ,corner-edge))
		      ((or lb rb) `(top-edge . ,top-edge))
		      ((or 1b 2b) `(top-edge . ,-border-width)))
		`(,(if (or ib tb bb) 'height 'width) . ,border-width)
		(cond ((or ib tb bb) `(right-edge . ,corner-edge))
		      ((or lb 1b) `(left-edge  . ,-border-width))
		      ((or rb 2b) `(right-edge . ,-border-width)))
		(cond (ib `(top-edge . ,-border-width))
		      (tb `(top-edge . ,titlebar-top))
		      (bb `(bottom-edge . ,-border-width))
		      ((or lb rb 1b 2b) `(bottom-edge . ,corner-edge)))
		`(class . ,(cond (lb 'left-border)
				 (rb 'right-border)
				 (t class)))))))

     (make-corner-horiz
      (lambda (class)
	"Return the horizontal section of the sizing corner of the specified class."
	(let ((il (eq class 'title-left-corner)) (ir (eq class 'title-right-corner))
	      (tl (eq class 'top-left-corner)) (tr (eq class 'top-right-corner))
	      (bl (eq class 'bottom-left-corner)) (br (eq class 'bottom-right-corner)))
	  (list `(background . ,corner-colors)
		(cond ((or il tl bl) `(left-edge . ,-corner-border-width))
		      ((or ir tr br) `(right-edge . ,-corner-border-width)))
		`(width . ,corner-side)
		`(height . ,corner-border-width)
		(cond ((or il ir) `(top-edge . ,corner-top))
		      ((or tl tr) `(top-edge . ,-corner-border-width))
		      ((or bl br) `(bottom-edge . ,-corner-border-width)))
		`(class . ,(cond (il 'top-left-corner)
				 (ir 'top-right-corner)
				 (t class)))))))

      (make-corner-vert
       (lambda (class)
	 "Return the vertical section of the sizing corner of the specified class."
	 (let ((il (eq class 'title-left-corner)) (ir (eq class 'title-right-corner))
	       (tl (eq class 'top-left-corner)) (tr (eq class 'top-right-corner))
	       (bl (eq class 'bottom-left-corner)) (br (eq class 'bottom-right-corner)))
	   (list `(background . ,corner-colors)
		 (cond ((or il ir) `(top-edge . ,corner-top))
		       ((or tl tr) `(top-edge . ,-border-width))
		       ((or bl br) `(bottom-edge . ,-corner-border-width)))
		 `(height . ,corner-side)
		 `(width . ,corner-border-width)
		 (cond ((or il tl bl) `(left-edge . ,-corner-border-width))
		       ((or ir tr br) `(right-edge . ,-corner-border-width)))
		 `(class . ,class)))))

     ;; frame layout
     (frame `(
	      ;; titlebar (close button background)
              ((background . ,titlebar-left)
               (top-edge . -25)
               (left-edge . 0)
	       (width . 17)
               (cursor . left_ptr)
               (class . title))

	      ;; titlebar (title area)
	      ((background . ,titlebar-left)
	       (foreground . ,title-colors)
	       (text . ,window-name)
	       (x-justify . center)
	       (y-justify . center)
	       (top-edge . -25)
	       (left-edge . ,closebox-offset)
	       (width . ,(lambda (w) (title-width w)))
               (cursor . left_ptr)
	       (class . title))

	      ;; titlebar (middle)
	      ((background . ,middle)
	       (left-edge . ,(lambda (w) (+ (closebox-offset) (title-width w))))
	       (top-edge . -25)
               (cursor . left_ptr)
	       (class . title))

	      ;; titlebar (right side)
	      ((background . ,titlebar-right)
	       (top-edge . -25)
	       (left-edge . ,(lambda (w) (+ (title-width w) (closebox-offset) 25)))
	       (right-edge . 0)
               (cursor . left_ptr)
	       (class . title))

              ;; close button
              ((background . ,close)
	       (foreground . ,title-colors)
               (top-edge . -25)
               (left-edge . 3)
               (cursor . left_ptr)
               (class . close-button))

              ;; maximize button
              ((background . ,maximize)
               (top-edge . -25)
               (right-edge . 20)
               (cursor . left_ptr)
               (class . maximize-button))

              ;; iconify button
              ((background . ,iconify)
               (top-edge . -25)
               (right-edge . 37)
               (cursor . left_ptr)
               (class . iconify-button))

              ;; shade button
              ((background . ,shade)
               (top-edge . -25)
               (right-edge . 2)
               (cursor . left_ptr)
               (class . shade-button))

              ;; resize corner
              ((background . ,corner)
               (bottom-edge . 0)
               (right-edge . 0)
               (cursor . left_ptr)
               (class . bottom-right-corner))

 	      ;; top border
	      ,(make-border 'top-border)

 	      ;; left border
	      ,(make-border 'full-left-border)

	      ;; right border
	      ,(make-border 'full-right-border)

	      ;; bottom border
	      ,(make-border 'bottom-border)

 	      ;; top left corner
	      ,(make-corner-horiz 'title-left-corner)
	      ,(make-corner-vert 'title-left-corner)

	      ;; top right corner
	      ,(make-corner-horiz 'title-right-corner)
	      ,(make-corner-vert 'title-right-corner)

	      ;; bottom left corner
	      ,(make-corner-horiz 'bottom-left-corner)
	      ,(make-corner-vert 'bottom-left-corner)
	      
	      ;; bottom right corner
	      ,(make-corner-horiz 'bottom-right-corner)
	      ,(make-corner-vert 'bottom-right-corner)

       ))

       ;; shaped: titlebar only
       ;; transient: borders only
       ;; shaped transient: top border only

       (shaped-frame `(
	      ;; titlebar (left side)
	      ((background . ,titlebar-left)
	       (foreground . ,title-colors)
	       (text . ,window-name)
	       (x-justify . center)
	       (y-justify . center)
	       (top-edge . -25)
	       (left-edge . 0)
	       (width . ,(lambda (w) (title-width w)))
               (cursor . left_ptr)
	       (class . title))

	      ;; middle
	      ((background . ,middle)
	       (left-edge . ,(lambda (w) (title-width w)))
	       (top-edge . -25)
               (cursor . left_ptr)
	       (class . title))

	      ;; titlebar (right side)
	      ((background . ,titlebar-right)
	       (top-edge . -25)
	       (left-edge . ,(lambda (w) (+ (title-width w) 25)))
	       (right-edge . 0)
               (cursor . left_ptr)
	       (class . title))

              ;; shade button
              ((background . ,shade)
               (top-edge . -25)
               (right-edge . 2)
               (cursor . left_ptr)
               (class . shade-button))
      ))

     (transient-frame `(
              ;; resize corner
              ((background . ,corner)
               (bottom-edge . 0)
               (right-edge . 0)
               (cursor . left_ptr)
               (class . bottom-right-corner))

 	      ;; top border (functions as titlebar too)
	      ,(cons '(cursor . left_ptr) (make-border 'title))

 	      ;; left border
	      ,(make-border 'left-border)

	      ;; right border
	      ,(make-border 'right-border)

	      ;; bottom border
	      ,(make-border 'bottom-border)

 	      ;; top left corner
	      ,(make-corner-horiz 'top-left-corner)
	      ,(make-corner-vert 'top-left-corner)

	      ;; top right corner
	      ,(make-corner-horiz 'top-right-corner)
	      ,(make-corner-vert 'top-right-corner)

	      ;; bottom left corner
	      ,(make-corner-horiz 'bottom-left-corner)
	      ,(make-corner-vert 'bottom-left-corner)
	      
	      ;; bottom right corner
	      ,(make-corner-horiz 'bottom-right-corner)
	      ,(make-corner-vert 'bottom-right-corner)
     ))

     (shaped-transient-frame `(
 	      ;; top border (functions as titlebar too)
	      ,(cons '(cursor . left_ptr) (make-border 'title))
     ))
  )
  
  (def-frame-class shade-button '((cursor . left_ptr))
      (bind-keys shade-button-keymap "Button1-Off" 'toggle-window-shaded)
      (bind-keys shade-button-keymap "Button3-Click1" 'popup-window-menu))

  (add-frame-style 's.t.o.3
		   (lambda (w type)
		     (case type
		       ((default)                           frame)
		       ((shaped shaded)                     shaped-frame)
		       ((transient)                         transient-frame)
		       ((shaped-transient shaded-transient) shaped-transient-frame)
		       ((unframed)                          nil-frame))))
  
  (mapc (lambda (sym)
	  (custom-set-property sym ':after-set rebuild))
	'(sto3:border-color sto3:border-width sto3:use-corner sto3:corner-color-enabled
	  sto3:corner-color-disabled sto3:corner-side sto3:corner-border-width
	  sto3:closebox-offset))
  
  (init)
)

(call-after-property-changed 'WM_NAME
  (lambda (w)
    (if (eq (window-get w 'current-frame-style) 's.t.o.3) (rebuild-frame w))))